/*=======================================================================

  Program:   
  Module:    $RCSfile: $
  Language:  C++
  Date:      $Date: $
  Version:   $Revision: $
  Author:    $Author: $
  Comment:   Adapted from the VTK examples

=========================================================================*/



// VTK includes
#include <vtkActor.h>
#include <vtkCamera.h>
#include <vtkCylinder.h>
#include <vtkExtractPolyDataGeometry.h>
#include <vtkGlyph3D.h>
#include <vtkPolyDataMapper.h>
#include <vtkProperty.h>
#include <vtkRenderer.h>
#include <vtkRenderWindow.h>
#include <vtkRenderWindowInteractor.h>
#include <vtkSmartPointer.h>
#include <vtkSphereSource.h>
#include <vtkTransform.h>


/** Demonstrate how to extract polygonal cells with an implicit function.
*/

int main(int argc, char* argv[])
{

 // Create a sphere source and actor
  vtkSmartPointer<vtkSphereSource> sphere = 
    vtkSmartPointer<vtkSphereSource>::New();
  sphere->SetThetaResolution(8);
  sphere->SetPhiResolution(16);
  sphere->SetRadius(2);

  // Extraction stuff
  vtkSmartPointer<vtkTransform> t = 
    vtkSmartPointer<vtkTransform>::New();
  t->RotateX(90);

  vtkSmartPointer<vtkCylinder> cylfunc = 
    vtkSmartPointer<vtkCylinder>::New();
  cylfunc->SetRadius(1.25);
  cylfunc->SetTransform(t);

  vtkSmartPointer<vtkExtractPolyDataGeometry> extract = 
    vtkSmartPointer<vtkExtractPolyDataGeometry>::New();
  extract->SetInputConnection(sphere->GetOutputPort());
  extract->SetImplicitFunction(cylfunc);
  extract->ExtractBoundaryCellsOn();
#if VTK_MAJOR_VERSION > 5
  extract->PassPointsOn();
#endif


  vtkSmartPointer<vtkPolyDataMapper> sphereMapper = 
    vtkSmartPointer<vtkPolyDataMapper>::New();
  sphereMapper->SetInputConnection(extract->GetOutputPort());
  sphereMapper->GlobalImmediateModeRenderingOn();

  vtkSmartPointer<vtkActor> sphereActor = 
    vtkSmartPointer<vtkActor>::New();
  sphereActor->SetMapper(sphereMapper);


  // Extraction stuff - now cull points
  vtkSmartPointer<vtkExtractPolyDataGeometry> extract2 = 
    vtkSmartPointer<vtkExtractPolyDataGeometry>::New();

  extract2->SetInputConnection(sphere->GetOutputPort());
  extract2->SetImplicitFunction(cylfunc);
  extract2->ExtractBoundaryCellsOn();
#if VTK_MAJOR_VERSION > 5
  extract2->PassPointsOff();
#endif

  vtkSmartPointer<vtkPolyDataMapper> sphereMapper2 = 
    vtkSmartPointer<vtkPolyDataMapper>::New();
  sphereMapper2->SetInputConnection(extract2->GetOutputPort());

  vtkSmartPointer<vtkActor> sphereActor2 = 
    vtkSmartPointer<vtkActor>::New();
  sphereActor2->SetMapper(sphereMapper2);
  sphereActor2->AddPosition(2.5, 0, 0);
  sphereActor2->GetProperty()->SetColor(1.0, 0.0, 0.0);

  // Put some glyphs on the points
  vtkSmartPointer<vtkSphereSource> glyphSphere = 
    vtkSmartPointer<vtkSphereSource>::New();
  glyphSphere->SetRadius(0.05);

  vtkSmartPointer<vtkGlyph3D> glyph = 
    vtkSmartPointer<vtkGlyph3D>::New();
  glyph->SetInputConnection(extract->GetOutputPort());
  glyph->SetSourceConnection(glyphSphere->GetOutputPort());
  glyph->SetScaleModeToDataScalingOff();

  vtkSmartPointer<vtkPolyDataMapper> glyphMapper = 
    vtkSmartPointer<vtkPolyDataMapper>::New();
  glyphMapper->SetInputConnection(glyph->GetOutputPort());

  vtkSmartPointer<vtkActor> glyphActor = 
    vtkSmartPointer<vtkActor>::New();
  glyphActor->SetMapper(glyphMapper);

  vtkSmartPointer<vtkGlyph3D> glyph2 = 
    vtkSmartPointer<vtkGlyph3D>::New();

  glyph2->SetInputConnection(extract2->GetOutputPort()),
  glyph2->SetSourceConnection(glyphSphere->GetOutputPort()),
  glyph2->SetScaleModeToDataScalingOff();

  vtkSmartPointer<vtkPolyDataMapper> glyphMapper2 = 
    vtkSmartPointer<vtkPolyDataMapper>::New();
  glyphMapper2->SetInputConnection(glyph2->GetOutputPort());

  vtkSmartPointer<vtkActor> glyphActor2 = 
    vtkSmartPointer<vtkActor>::New();
  glyphActor2->SetMapper(glyphMapper2);
  glyphActor2->AddPosition(2.5, 0, 0);


  // Create the RenderWindow, Renderer and both Actors
  vtkSmartPointer<vtkRenderer> ren1 = 
    vtkSmartPointer<vtkRenderer>::New();

  vtkSmartPointer<vtkRenderWindow> renWin = 
    vtkSmartPointer<vtkRenderWindow>::New();
  renWin->AddRenderer(ren1);
  renWin->SetWindowName("vtk - extractPolyData");

  vtkSmartPointer<vtkRenderWindowInteractor> iren = 
    vtkSmartPointer<vtkRenderWindowInteractor>::New();
  iren->SetRenderWindow(renWin);

  // Add the actors to the renderer, set the background and size
  ren1->AddActor(sphereActor);
  ren1->AddActor(glyphActor);
  ren1->AddActor(sphereActor2);
  ren1->AddActor(glyphActor2);
  ren1->ResetCamera();
  ren1->GetActiveCamera()->Azimuth(30);
  ren1->SetBackground(0.0, 0.0, 0.0);
  renWin->SetSize(300, 300);
  renWin->Render();

  // render the image

  iren->Initialize();
  iren->Start();

  return EXIT_SUCCESS;
}
